@@ -1,5 +1,5 @@
 /*  Audacious
- *  Copyright (C) 2005-2007  Audacious team.
+ *  Copyright (C) 2005-2008  Audacious team.
  *
  *  BMP - Cross-platform multimedia player
  *  Copyright (C) 2003-2004  BMP development team.
@@ -15,8 +15,8 @@
  *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in 
- *    the documentation and/or other materials provided with the 
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
  *    distribution.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
@@ -32,18 +32,22 @@
  * SUCH DAMAGE.
  */
 
-#ifndef BMP_PLUGIN_H
-#define BMP_PLUGIN_H
+#ifndef AUDACIOUS_PLUGIN_H
+#define AUDACIOUS_PLUGIN_H
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include "audacious/vfs.h"
-#include "audacious/tuple.h"
-#include "audacious/tuple_formatter.h"
+#include "libaudcore/vfs.h"
+#include "libaudcore/tuple.h"
+#include "libaudcore/tuple_formatter.h"
 #include "audacious/eventqueue.h"
 #include "audacious/configdb.h"
 #include "audacious/playlist_container.h"
 #include "audacious/main.h"
+#include "audacious/preferences.h"
+#include "audacious/interface.h"
+#include "audacious/equalizer_preset.h"
+#include "libSAD/libSAD.h"
 
 #define PLUGIN(x)         ((Plugin *)(x))
 #define INPUT_PLUGIN(x)   ((InputPlugin *)(x))
@@ -56,20 +60,71 @@
 #define LOWLEVEL_PLUGIN(x) ((LowlevelPlugin *)(x))
 
 #define __AUDACIOUS_NEWVFS__
-#define __AUDACIOUS_PLUGIN_API__ 7
-#define __AUDACIOUS_INPUT_PLUGIN_API__ 7
+#define __AUDACIOUS_PLUGIN_API__ 8
+#define __AUDACIOUS_INPUT_PLUGIN_API__ 8
 
 typedef enum {
     FMT_U8,
     FMT_S8,
+
     FMT_U16_LE,
     FMT_U16_BE,
     FMT_U16_NE,
     FMT_S16_LE,
     FMT_S16_BE,
-    FMT_S16_NE
+    FMT_S16_NE,
+
+    /* added in Audacious 1.5 */
+    FMT_U24_LE, /* stored in lower 3 bytes of 32-bit value, highest byte must be always set to 0 --asphyx */
+    FMT_U24_BE,
+    FMT_U24_NE,
+    FMT_S24_LE,
+    FMT_S24_BE,
+    FMT_S24_NE,
+
+    FMT_U32_LE,
+    FMT_U32_BE,
+    FMT_U32_NE,
+    FMT_S32_LE,
+    FMT_S32_BE,
+    FMT_S32_NE,
+
+    FMT_FLOAT,
+    FMT_FIXED32 /* equivalent of libmad's mad_fixed_t explained below */
 } AFormat;
 
+/* From mad.h:
+ *
+ * Fixed-point format: 0xABBBBBBB
+ * A == whole part      (sign + 3 bits)
+ * B == fractional part (28 bits)
+ *
+ * Values are signed two's complement, so the effective range is:
+ * 0x80000000 to 0x7fffffff
+ *       -8.0 to +7.9999999962747097015380859375
+ *
+ * The smallest representable value is:
+ * 0x00000001 == 0.0000000037252902984619140625 (i.e. about 3.725e-9)
+ *
+ * 28 bits of fractional accuracy represent about
+ * 8.6 digits of decimal accuracy.
+ *
+ * Fixed-point numbers can be added or subtracted as normal
+ * integers, but multiplication requires shifting the 64-bit result
+ * from 56 fractional bits back to 28 (and rounding.)
+ */
+
+#define __AUDACIOUS_ASSUMED_MAD_F_FRACBITS__ 28 /* useful for build time check for plugins linked against libmad, i.e. madplug */
+
+#define FMT_SIZEOF(a) ( \
+    (a == FMT_S8 || a == FMT_U8) ? sizeof(gint8) : (                                                                   \
+    (a == FMT_S16_NE || a == FMT_S16_LE || a == FMT_S16_BE || a == FMT_U16_NE || a == FMT_U16_LE || a == FMT_U16_BE) ? sizeof(gint16) : ( \
+    (a == FMT_S24_NE || a == FMT_S24_LE || a == FMT_S24_BE || a == FMT_U24_NE || a == FMT_U24_LE || a == FMT_U24_BE || \
+     a == FMT_S32_NE || a == FMT_S32_LE || a == FMT_S32_BE || a == FMT_U32_NE || a == FMT_U32_LE || a == FMT_U32_BE || \
+     a == FMT_FIXED32) ? sizeof(gint32) : sizeof(float))))
+
+#define FMT_FRACBITS(a) ( (a) == FMT_FIXED32 ? __AUDACIOUS_ASSUMED_MAD_F_FRACBITS__ : 0 )
+
 typedef enum {
     INPUT_VIS_ANALYZER,
     INPUT_VIS_SCOPE,
@@ -94,6 +149,13 @@ typedef struct {
     InputPlugin *ip;
 } ProbeResult;
 
+typedef struct {
+    gfloat track_gain; /* in dB !!! --asphyx */
+    gfloat track_peak;
+    gfloat album_gain;
+    gfloat album_peak;
+} ReplayGainInfo;
+
 typedef GHashTable INIFile;
 
 #include "audacious/playlist.h"
@@ -101,9 +163,6 @@ typedef GHashTable INIFile;
 #include "audacious/mime.h"
 #include "audacious/custom_uri.h"
 #include "audacious/hook.h"
-#include "audacious/xconvert.h"
-#include "audacious/ui_plugin_menu.h"
-#include "audacious/formatter.h"
 #include "audacious/flow.h"
 
 #define PLUGIN_COMMON_FIELDS		\
@@ -115,7 +174,7 @@ typedef GHashTable INIFile;
     void (*about) (void);		\
     void (*configure) (void);		\
     gboolean enabled;
-	
+
 
 /*
  * The v2 Module header.
@@ -137,6 +196,7 @@ typedef struct {
     GeneralPlugin **gp_list;
     VisPlugin **vp_list;
     DiscoveryPlugin **dp_list;
+    Interface *interface;
 } PluginHeader;
 
 #define PLUGIN_MAGIC 0x8EAC8DE2
@@ -197,52 +257,52 @@ struct _AudaciousFuncTableV1 {
     VFSFile *(*vfs_buffered_file_release_live_fd)(VFSFile *fd);
 
     /* ConfigDb */
-    ConfigDb *(*cfg_db_open)(void);
-    void (*cfg_db_close)(ConfigDb *db);
+    mcs_handle_t *(*cfg_db_open)(void);
+    void (*cfg_db_close)(mcs_handle_t *db);
 
-    gboolean (*cfg_db_get_string)(ConfigDb *db,
+    gboolean (*cfg_db_get_string)(mcs_handle_t *db,
                                   const gchar *section,
                                   const gchar *key,
                                   gchar **value);
-    gboolean (*cfg_db_get_int)(ConfigDb *db,
+    gboolean (*cfg_db_get_int)(mcs_handle_t *db,
                                const gchar *section,
                                const gchar *key,
                                gint *value);
-    gboolean (*cfg_db_get_bool)(ConfigDb *db,
+    gboolean (*cfg_db_get_bool)(mcs_handle_t *db,
                                 const gchar *section,
                                 const gchar *key,
                                 gboolean *value);
-    gboolean (*cfg_db_get_float)(ConfigDb *db,
+    gboolean (*cfg_db_get_float)(mcs_handle_t *db,
                                  const gchar *section,
                                  const gchar *key,
                                  gfloat *value);
-    gboolean (*cfg_db_get_double)(ConfigDb *db,
+    gboolean (*cfg_db_get_double)(mcs_handle_t *db,
                                   const gchar *section,
                                   const gchar *key,
                                   gdouble *value);
 
-    void (*cfg_db_set_string)(ConfigDb *db,
+    void (*cfg_db_set_string)(mcs_handle_t *db,
                               const gchar *section,
                               const gchar *key,
                               const gchar *value);
-    void (*cfg_db_set_int)(ConfigDb *db,
+    void (*cfg_db_set_int)(mcs_handle_t *db,
                            const gchar *section,
                            const gchar *key,
                            gint value);
-    void (*cfg_db_set_bool)(ConfigDb *db,
+    void (*cfg_db_set_bool)(mcs_handle_t *db,
                             const gchar *section,
                             const gchar *key,
                             gboolean value);
-    void (*cfg_db_set_float)(ConfigDb *db,
+    void (*cfg_db_set_float)(mcs_handle_t *db,
                              const gchar *section,
                              const gchar *key,
                              gfloat value);
-    void (*cfg_db_set_double)(ConfigDb *db,
+    void (*cfg_db_set_double)(mcs_handle_t *db,
                               const gchar *section,
                               const gchar *key,
                               gdouble value);
 
-    void (*cfg_db_unset_key)(ConfigDb *db,
+    void (*cfg_db_unset_key)(mcs_handle_t *db,
                              const gchar *section,
                              const gchar *key);
 
@@ -304,6 +364,9 @@ struct _AudaciousFuncTableV1 {
     gchar *(*util_get_localdir)(void);
     void (*util_menu_main_show)(gint x, gint y, guint button, guint time);
 
+    gpointer (*smart_realloc)(gpointer ptr, gsize *size);
+    SAD_sample_format (*sadfmt_from_afmt)(AFormat fmt);
+
     /* INI funcs */
     INIFile *(*open_ini_file)(const gchar *filename);
     void (*close_ini_file)(INIFile *key_file);
@@ -460,7 +523,7 @@ struct _AudaciousFuncTableV1 {
 
     /* state vars */
     InputPluginData *ip_state;
-    BmpConfig *_cfg;
+    AudConfig *_cfg;
 
     /* hook API */
     void (*hook_register)(const gchar *name);
@@ -468,17 +531,6 @@ struct _AudaciousFuncTableV1 {
     gint (*hook_dissociate)(const gchar *name, HookFunction func);
     void (*hook_call)(const gchar *name, gpointer hook_data);
 
-    /* xconvert API */
-    struct xmms_convert_buffers *(*xmms_convert_buffers_new)(void);
-    void (*xmms_convert_buffers_free)(struct xmms_convert_buffers *buf);
-    void (*xmms_convert_buffers_destroy)(struct xmms_convert_buffers *buf);
-    convert_func_t (*xmms_convert_get_func)(AFormat output, AFormat input);
-    convert_channel_func_t (*xmms_convert_get_channel_func)(AFormat fmt,
-                                                     int output,
-                                                     int input);
-    convert_freq_func_t (*xmms_convert_get_frequency_func)(AFormat fmt,
-                                                    int channels);
-
     /* PluginMenu API */
     gint (*menu_plugin_item_add)(gint, GtkWidget *);
     gint (*menu_plugin_item_remove)(gint, GtkWidget *);
@@ -497,6 +549,7 @@ struct _AudaciousFuncTableV1 {
     void (*drct_activate)(void);
 
     /* DRCT API: playback */
+    void (*drct_initiate) ( void );
     void (*drct_play) ( void );
     void (*drct_pause) ( void );
     void (*drct_stop) ( void );
@@ -505,6 +558,7 @@ struct _AudaciousFuncTableV1 {
     gboolean (*drct_get_stopped)( void );
     void (*drct_get_info)( gint *rate, gint *freq, gint *nch);
     gint (*drct_get_time )( void );
+    gint (*drct_get_length )( void );
     void (*drct_seek) ( guint pos );
     void (*drct_get_volume)( gint *vl, gint *vr );
     void (*drct_set_volume)( gint vl, gint vr );
@@ -542,14 +596,6 @@ struct _AudaciousFuncTableV1 {
     gint (*drct_pq_get_position)( gint pos );
     gint (*drct_pq_get_queue_position)( gint pos );
 
-    /* Formatter API */
-    Formatter *(*formatter_new)(void);
-    void (*formatter_destroy)(Formatter * formatter);
-    void (*formatter_associate)(Formatter * formatter, guchar id,
-                                gchar * value);
-    void (*formatter_dissociate)(Formatter * formatter, guchar id);
-    gchar *(*formatter_format)(Formatter * formatter, gchar * format);
-
     gint (*prefswin_page_new)(GtkWidget *container, gchar *name, gchar *imgurl);
     void (*prefswin_page_destroy)(GtkWidget *container);
 
@@ -565,6 +611,7 @@ struct _AudaciousFuncTableV1 {
 
     /* InputPlayback */
     InputPlayback *(*playback_new)(void);
+    void (*playback_free)(InputPlayback *);
     void (*playback_run)(InputPlayback *);
 
     /* Flows */
@@ -574,20 +621,46 @@ struct _AudaciousFuncTableV1 {
     void (*flow_link_element)(Flow *flow, FlowFunction func);
     void (*flow_unlink_element)(Flow *flow, FlowFunction func);
     void (*effect_flow)(FlowContext *context);
-    void (*iir_flow)(FlowContext *context);
     void (*volumecontrol_flow)(FlowContext *context);
-    
-    /* Dock */
-    GList *(*get_dock_window_list)(void);
-    GList *(*dock_add_window)(GList * window_list, GtkWindow * window);
-    GList *(*dock_remove_window)(GList * window_list, GtkWindow * window);
-    void (*dock_move_press)(GList * window_list, GtkWindow * w,
-                         GdkEventButton * event, gboolean move_list);
-    void (*dock_move_motion)(GtkWindow * w, GdkEventMotion * event);
-    void (*dock_move_release)(GtkWindow * w);
-    gboolean (*dock_is_moving)(GtkWindow * w);
 
     GList *(*get_output_list)(void);
+
+    void (*input_get_volume)(gint * l, gint * r);
+
+    gchar *(*construct_uri)(gchar *string, const gchar *playlist_name);
+    gchar *(*uri_to_display_basename)(const gchar * uri);
+    gchar *(*uri_to_display_dirname)(const gchar * uri);
+
+    void (*set_pvt_data)(Plugin * plugin, gpointer data);
+    gpointer (*get_pvt_data)(void);
+
+    void (*event_queue)(const gchar *name, gpointer user_data);
+
+    void (*calc_mono_freq)(gint16 dest[2][256], gint16 src[2][512], gint nch);
+    void (*calc_mono_pcm)(gint16 dest[2][512], gint16 src[2][512], gint nch);
+    void (*calc_stereo_pcm)(gint16 dest[2][512], gint16 src[2][512], gint nch);
+
+    void (*create_widgets)(GtkBox *box, PreferencesWidget *widgets, gint amt);
+
+    GList *(*equalizer_read_presets)(const gchar * basename);
+    void (*equalizer_write_preset_file)(GList * list, const gchar * basename);
+    GList *(*import_winamp_eqf)(VFSFile * file);
+    void (*save_preset_file)(EqualizerPreset *preset, const gchar * filename);
+    EqualizerPreset *(*equalizer_read_aud_preset)(const gchar * filename);
+    EqualizerPreset *(*load_preset_file)(const gchar *filename);
+
+    /* VFS endianess helper functions */
+    gboolean (*vfs_fget_le16)(guint16 *value, VFSFile *stream);
+    gboolean (*vfs_fget_le32)(guint32 *value, VFSFile *stream);
+    gboolean (*vfs_fget_le64)(guint64 *value, VFSFile *stream);
+    gboolean (*vfs_fget_be16)(guint16 *value, VFSFile *stream);
+    gboolean (*vfs_fget_be32)(guint32 *value, VFSFile *stream);
+    gboolean (*vfs_fget_be64)(guint64 *value, VFSFile *stream);
+    void (*output_plugin_cleanup)(void);
+    void (*output_plugin_reinit)(void);
+
+    /* PluginMenu API */
+    GtkWidget * (* get_plugin_menu) (int id);
 };
 
 /* Convenience macros for accessing the public API. */
@@ -621,21 +694,15 @@ struct _AudaciousFuncTableV1 {
 #define aud_vfs_buffered_file_new_from_uri	_audvt->vfs_buffered_file_new_from_uri
 #define aud_vfs_buffered_file_release_live_fd	_audvt->vfs_buffered_file_release_live_fd
 
-/* XXX: deprecation warnings */
-#define bmp_cfg_db_open			_audvt->cfg_db_open
-#define bmp_cfg_db_close		_audvt->cfg_db_close
-#define bmp_cfg_db_set_string		_audvt->cfg_db_set_string
-#define bmp_cfg_db_set_int		_audvt->cfg_db_set_int
-#define bmp_cfg_db_set_bool		_audvt->cfg_db_set_bool
-#define bmp_cfg_db_set_float		_audvt->cfg_db_set_float
-#define bmp_cfg_db_set_double		_audvt->cfg_db_set_double
-#define bmp_cfg_db_get_string		_audvt->cfg_db_get_string
-#define bmp_cfg_db_get_int		_audvt->cfg_db_get_int
-#define bmp_cfg_db_get_bool		_audvt->cfg_db_get_bool
-#define bmp_cfg_db_get_float		_audvt->cfg_db_get_float
-#define bmp_cfg_db_get_double		_audvt->cfg_db_get_double
-#define bmp_cfg_db_unset_key		_audvt->cfg_db_unset_key
+#define aud_vfs_fget_le16		_audvt->vfs_fget_le16
+#define aud_vfs_fget_le32		_audvt->vfs_fget_le32
+#define aud_vfs_fget_le64		_audvt->vfs_fget_le64
+#define aud_vfs_fget_be16		_audvt->vfs_fget_be16
+#define aud_vfs_fget_be32		_audvt->vfs_fget_be32
+#define aud_vfs_fget_be64		_audvt->vfs_fget_be64
 
+/* XXX: deprecation warnings */
+#define ConfigDb mcs_handle_t		/* Alias for compatibility -- ccr */
 #define aud_cfg_db_open			_audvt->cfg_db_open
 #define aud_cfg_db_close		_audvt->cfg_db_close
 #define aud_cfg_db_set_string		_audvt->cfg_db_set_string
@@ -678,6 +745,8 @@ struct _AudaciousFuncTableV1 {
 #define aud_info_dialog			_audvt->util_info_dialog
 #define audacious_info_dialog		_audvt->util_info_dialog
 #define aud_get_gentitle_format		_audvt->get_gentitle_format
+#define aud_smart_realloc               _audvt->smart_realloc
+#define aud_sadfmt_from_afmt            _audvt->sadfmt_from_afmt
 
 #define aud_escape_shell_chars		_audvt->escape_shell_chars
 #define aud_str_append			_audvt->str_append
@@ -816,13 +885,6 @@ struct _AudaciousFuncTableV1 {
 #define aud_read_ini_string			_audvt->read_ini_string
 #define aud_read_ini_array			_audvt->read_ini_array
 
-#define aud_convert_buffers_new			_audvt->xmms_convert_buffers_new
-#define aud_convert_buffers_free		_audvt->xmms_convert_buffers_free
-#define aud_convert_buffers_destroy		_audvt->xmms_convert_buffers_destroy
-#define aud_convert_get_func			_audvt->xmms_convert_get_func
-#define aud_convert_get_channel_func		_audvt->xmms_convert_get_channel_func
-#define aud_convert_get_frequency_func		_audvt->xmms_convert_get_frequency_func
-
 #define audacious_menu_plugin_item_add		_audvt->menu_plugin_item_add
 #define audacious_menu_plugin_item_remove	_audvt->menu_plugin_item_remove
 #define aud_menu_plugin_item_add		_audvt->menu_plugin_item_add
@@ -840,6 +902,7 @@ struct _AudaciousFuncTableV1 {
 #define audacious_drct_set_skin			_audvt->drct_set_skin
 #define audacious_drct_activate			_audvt->drct_activate
 
+#define audacious_drct_initiate                 _audvt->drct_initiate
 #define audacious_drct_play			_audvt->drct_play
 #define audacious_drct_pause			_audvt->drct_pause
 #define audacious_drct_stop			_audvt->drct_stop
@@ -848,6 +911,7 @@ struct _AudaciousFuncTableV1 {
 #define audacious_drct_get_stopped		_audvt->drct_get_stopped
 #define audacious_drct_get_info			_audvt->drct_get_info
 #define audacious_drct_get_time			_audvt->drct_get_time
+#define audacious_drct_get_length               _audvt->drct_get_length
 #define audacious_drct_seek			_audvt->drct_seek
 #define audacious_drct_get_volume		_audvt->drct_get_volume
 #define audacious_drct_set_volume		_audvt->drct_set_volume
@@ -883,12 +947,6 @@ struct _AudaciousFuncTableV1 {
 #define audacious_drct_pq_get_position		_audvt->drct_pq_get_position
 #define audacious_drct_pq_get_queue_position	_audvt->drct_pq_get_queue_position
 
-#define aud_formatter_new			_audvt->formatter_new
-#define aud_formatter_destroy			_audvt->formatter_destroy
-#define aud_formatter_associate			_audvt->formatter_associate
-#define aud_formatter_dissociate		_audvt->formatter_dissociate
-#define aud_formatter_format			_audvt->formatter_format
-
 #define aud_prefswin_page_new			_audvt->prefswin_page_new
 #define aud_prefswin_page_destroy		_audvt->prefswin_page_destroy
 
@@ -904,14 +962,13 @@ struct _AudaciousFuncTableV1 {
 
 #define aud_playback_new			_audvt->playback_new
 #define aud_playback_run			_audvt->playback_run
-#define aud_playback_free(x)			g_slice_free(InputPlayback, (x))
+#define aud_playback_free(x)			_audvt->playback_free
 
 #define aud_flow_execute			_audvt->flow_execute
 #define aud_flow_new				_audvt->flow_new
 #define aud_flow_link_element			_audvt->flow_link_element
 #define aud_flow_unlink_element			_audvt->flow_unlink_element
 #define aud_effect_flow				_audvt->effect_flow
-#define aud_iir_flow				_audvt->iir_flow
 #define aud_volumecontrol_flow			_audvt->volumecontrol_flow
 #define aud_flow_destroy(flow)			mowgli_object_unref(flow)
 
@@ -927,6 +984,34 @@ struct _AudaciousFuncTableV1 {
 
 #define aud_get_output_list			_audvt->get_output_list
 
+#define aud_input_get_volume			_audvt->input_get_volume
+
+#define aud_construct_uri        _audvt->construct_uri
+#define aud_uri_to_display_basename _audvt->uri_to_display_basename
+#define aud_uri_to_display_dirname _audvt->uri_to_display_dirname
+
+#define aud_set_pvt_data			_audvt->set_pvt_data
+#define aud_get_pvt_data			_audvt->get_pvt_data
+
+#define aud_event_queue             _audvt->event_queue
+
+#define aud_calc_mono_freq          _audvt->calc_mono_freq
+#define aud_calc_mono_pcm           _audvt->calc_mono_pcm
+#define aud_calc_stereo_pcm         _audvt->calc_stereo_pcm
+
+#define aud_create_widgets          _audvt->create_widgets
+
+#define aud_equalizer_read_presets  _audvt->equalizer_read_presets
+#define aud_equalizer_write_preset_file _audvt->equalizer_write_preset_file
+#define aud_import_winamp_eqf       _audvt->import_winamp_eqf
+#define aud_save_preset_file        _audvt->save_preset_file
+#define aud_equalizer_read_aud_preset _audvt->equalizer_read_aud_preset
+#define aud_load_preset_file        _audvt->load_preset_file
+#define aud_output_plugin_cleanup   _audvt->output_plugin_cleanup
+#define aud_output_plugin_reinit    _audvt->output_plugin_reinit
+
+#define aud_get_plugin_menu _audvt->get_plugin_menu
+
 #include "audacious/auddrct.h"
 
 /* for multi-file plugins :( */
@@ -963,6 +1048,9 @@ G_END_DECLS
 #define SIMPLE_DISCOVER_PLUGIN(name, dp_list) \
     DECLARE_PLUGIN(name, NULL, NULL, NULL, NULL, NULL, NULL, NULL, dp_list)
 
+#define SIMPLE_INTERFACE_PLUGIN(name, interface) \
+    DECLARE_PLUGIN(name, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, interface)
+
 /* Sadly, this is the most we can generalize out of the disparate
    plugin structs usable with typecasts - descender */
 struct _Plugin {
@@ -981,8 +1069,21 @@ struct _LowlevelPlugin {
     PLUGIN_COMMON_FIELDS
 };
 
+typedef enum {
+    OUTPUT_PLUGIN_INIT_FAIL,
+    OUTPUT_PLUGIN_INIT_NO_DEVICES,
+    OUTPUT_PLUGIN_INIT_FOUND_DEVICES,
+} OutputPluginInitStatus;
+
 struct _OutputPlugin {
-    PLUGIN_COMMON_FIELDS
+    gpointer handle;
+    gchar *filename;
+    gchar *description;
+    OutputPluginInitStatus (*init) (void);
+    void (*cleanup) (void);
+    void (*about) (void);
+    void (*configure) (void);
+    gboolean enabled;
 
     void (*get_volume) (gint * l, gint * r);
     void (*set_volume) (gint l, gint r);
@@ -999,6 +1100,8 @@ struct _OutputPlugin {
     gint (*written_time) (void);
 
     void (*tell_audio) (AFormat * fmt, gint * rate, gint * nch);
+
+    gint probe_priority;
 };
 
 struct _EffectPlugin {
@@ -1019,10 +1122,10 @@ struct _InputPlayback {
     gboolean eof;
 
     GThread *thread;
-    
+
     GMutex *pb_ready_mutex;
     GCond *pb_ready_cond;
-    gint pb_ready_val;    
+    gint pb_ready_val;
     gint (*set_pb_ready) (InputPlayback*);
 
     GMutex *pb_change_mutex;
@@ -1034,11 +1137,15 @@ struct _InputPlayback {
     gint freq;
     gint length;
     gchar *title;
-    
+
     void (*set_params) (InputPlayback *, gchar * title, gint length, gint rate, gint freq, gint nch);
     void (*set_title) (InputPlayback *, gchar * text);
 
     void (*pass_audio) (InputPlayback *, AFormat, gint, gint, gpointer, gint *);
+
+    /* added in Audacious 1.5 */
+    /* called by input plugin when RG info available --asphyx */
+    void (*set_replaygain_info) (InputPlayback *, ReplayGainInfo *);
 };
 
 struct _InputPlugin {
@@ -1052,8 +1159,6 @@ struct _InputPlugin {
     void (*pause) (InputPlayback * playback, gshort paused);
     void (*seek) (InputPlayback * playback, gint time);
 
-    void (*set_eq) (gint on, gfloat preamp, gfloat * bands);
-
     gint (*get_time) (InputPlayback * playback);
 
     gint (*get_volume) (gint * l, gint * r);
@@ -1067,12 +1172,8 @@ struct _InputPlugin {
     void (*get_song_info) (gchar * filename, gchar ** title, gint * length);
     void (*file_info_box) (gchar * filename);
 
-    OutputPlugin *output; /* deprecated */
-
     /* Added in Audacious 1.1.0 */
     Tuple *(*get_song_tuple) (gchar * filename);
-    void (*set_song_tuple) (Tuple * tuple);
-    void (*set_status_buffering) (gboolean status);
 
     /* Added in Audacious 1.3.0 */
     gint (*is_our_file_from_vfs) (gchar *filename, VFSFile *fd);
@@ -1085,6 +1186,21 @@ struct _InputPlugin {
     /* Added in Audacious 1.4.1 */
     gboolean have_subtune;
 
+    /* Added in Audacious 1.5.0 */
+    gboolean (*update_song_tuple)(Tuple *tuple, VFSFile *fd);
+    /*
+     * Plugin can provide this function for file metadata (aka tag) writing functionality
+     * in case when no reason to provide its own custom file info dialog. Thus in most cases.
+     *
+     * Some notes:
+     *
+     * 1. In current Audacious version, if plugin provides file_info_box(), the latter will be used in any case.
+     * 2. Each field in tuple means operation on one and only one tag's filed:
+     *     2.1. Set this field to appropriate value, if non-empty string or positive number provided.
+     *     2.2. Set this field to blank (or just delete, at plugins`s discretion), if empty string or negative number provided.
+     *
+     * -- eugene.
+     */
 };
 
 struct _GeneralPlugin {
@@ -1101,16 +1217,23 @@ struct _VisPlugin {
     void (*playback_start) (void);
     void (*playback_stop) (void);
     void (*render_pcm) (gint16 pcm_data[2][512]);
+
+    /* The range of intensities is 0 - 32767 (though theoretically it is
+     * possible for the FFT to result in bigger values, making the final
+     * intensity negative due to overflowing the 16bit signed integer.)
+     *
+     * If output is mono, only freq_data[0] is filled.
+     */
     void (*render_freq) (gint16 freq_data[2][256]);
 };
 
 struct _DiscoveryPlugin {
     PLUGIN_COMMON_FIELDS
 
-    GList *(*get_devices);  
+    GList *(*get_devices);
 };
 
 /* undefine the macro -- struct Plugin should be used instead. */
 #undef PLUGIN_COMMON_FIELDS
 
-#endif
+#endif /* AUDACIOUS_PLUGIN_H */
