@@ -18,9 +18,7 @@
  */
 
 #include "ui_playlist_manager.h"
-#include "ui_playlist.h"
 #include "playlist.h"
-#include "ui_main.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -66,12 +64,16 @@ playlist_manager_populate ( GtkListStore * store )
         gchar *pl_name = NULL;
         Playlist *playlist = (Playlist*)playlists->data;
 
-        PLAYLIST_LOCK(playlist);
+        if(playlist != active) //XXX the active playlist has been locked in playlist_new_from_selected()
+            PLAYLIST_LOCK(playlist);
+
         /* for each playlist, pick name and number of entries */
         pl_name = (gchar*)playlist_get_current_name( playlist );
         for (entries = playlist->entries; entries; entries = g_list_next(entries))
             entriesnum++;
-        PLAYLIST_UNLOCK(playlist);
+
+        if(playlist != active)
+            PLAYLIST_UNLOCK(playlist);
 
         /* update the tree model conservatively */
 
@@ -358,7 +360,6 @@ playlist_manager_ui_show ( void )
 
     playman_win = gtk_window_new( GTK_WINDOW_TOPLEVEL );
     gtk_window_set_type_hint( GTK_WINDOW(playman_win), GDK_WINDOW_TYPE_HINT_DIALOG );
-    gtk_window_set_transient_for( GTK_WINDOW(playman_win) , GTK_WINDOW(mainwin) );
     gtk_window_set_position( GTK_WINDOW(playman_win), GTK_WIN_POS_CENTER );
     gtk_window_set_title( GTK_WINDOW(playman_win), _("Playlist Manager") );
     gtk_container_set_border_width( GTK_CONTAINER(playman_win), 10 );
@@ -433,7 +434,7 @@ playlist_manager_ui_show ( void )
     /* button bar */
     playman_bbar_hbbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout( GTK_BUTTON_BOX(playman_bbar_hbbox) , GTK_BUTTONBOX_END );
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(playman_bbar_hbbox), 5);
+    gtk_box_set_spacing(GTK_BOX(playman_bbar_hbbox), 5);
     playman_bbar_bt_close = gtk_button_new_from_stock( GTK_STOCK_CLOSE );
     playman_bbar_bt_del = gtk_button_new_from_stock( GTK_STOCK_DELETE );
     playman_bbar_bt_new = gtk_button_new_from_stock( GTK_STOCK_NEW );
@@ -467,6 +468,8 @@ playlist_manager_ui_show ( void )
     g_object_unref( pl_store );
 
     gtk_widget_show_all( playman_win );
+
+    hook_associate("playlist update", (HookFunction) playlist_manager_update, NULL);
 }
 
 
